@@ -1,8 +1,7 @@
 /**************************************************************************
  *   proto.h  --  This file is part of GNU nano.                          *
  *                                                                        *
- *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,  *
- *   2008, 2009, 2010, 2011, 2013, 2014 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2017 Free Software Foundation, Inc.    *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
  *   it under the terms of the GNU General Public License as published    *
@@ -25,6 +24,7 @@
 #include "nano.h"
 
 /* All external variables.  See global.c for their descriptions. */
+
 #ifndef NANO_TINY
 extern volatile sig_atomic_t the_window_resized;
 #endif
@@ -43,19 +43,34 @@ extern bool as_an_at;
 extern int margin;
 extern int editwincols;
 
+#ifndef DISABLE_COLOR
+extern bool have_palette;
+#endif
+
+extern bool suppress_cursorpos;
+
 extern message_type lastmessage;
 
 extern filestruct *pletion_line;
 
+extern bool inhelp;
+extern char *title;
+
+extern int didfind;
+
 extern int controlleft;
 extern int controlright;
 extern int controlup;
 extern int controldown;
+extern int controlhome;
+extern int controlend;
 #ifndef NANO_TINY
 extern int shiftcontrolleft;
 extern int shiftcontrolright;
 extern int shiftcontrolup;
 extern int shiftcontroldown;
+extern int shiftcontrolhome;
+extern int shiftcontrolend;
 extern int shiftaltleft;
 extern int shiftaltright;
 extern int shiftaltup;
@@ -106,8 +121,6 @@ extern char *quoteerr;
 
 extern char *word_chars;
 
-extern bool nodelay_mode;
-
 extern char *answer;
 
 extern ssize_t tabsize;
@@ -162,17 +175,13 @@ extern char *homedir;
 
 typedef void (*functionptrtype)(void);
 
-/* All functions in browser.c. */
-#ifndef DISABLE_BROWSER
-char *do_browser(char *path);
+/* Most functions in browser.c. */
+#ifdef ENABLE_BROWSER
 char *do_browse_from(const char *inpath);
 void read_the_list(const char *path, DIR *dir);
 functionptrtype parse_browser_input(int *kbinput);
 void browser_refresh(void);
 void browser_select_dirname(const char *needle);
-int filesearch_init(void);
-void findnextfile(const char *needle);
-void filesearch_abort(void);
 void do_filesearch(void);
 void do_fileresearch(void);
 void do_first_file(void);
@@ -188,18 +197,13 @@ bool using_utf8(void);
 char *addstrings(char* str1, size_t len1, char* str2, size_t len2);
 bool is_byte(int c);
 bool is_alpha_mbchar(const char *c);
-bool is_alnum_mbchar(const char *c);
 bool is_blank_mbchar(const char *c);
 bool is_ascii_cntrl_char(int c);
-bool is_cntrl_char(int c);
 bool is_cntrl_mbchar(const char *c);
-bool is_punct_mbchar(const char *c);
 bool is_word_mbchar(const char *c, bool allow_punct);
-char control_rep(const signed char c);
 char control_mbrep(const char *c, bool isdata);
 int length_of_char(const char *c, int *width);
 int mbwidth(const char *c);
-int mb_cur_max(void);
 char *make_mbchar(long chr, int *chr_mb_len);
 int parse_mbchar(const char *buf, char *chr, size_t *col);
 size_t move_mbleft(const char *buf, size_t pos);
@@ -208,11 +212,9 @@ int mbstrcasecmp(const char *s1, const char *s2);
 int mbstrncasecmp(const char *s1, const char *s2, size_t n);
 char *mbstrcasestr(const char *haystack, const char *needle);
 char *revstrstr(const char *haystack, const char *needle,
-	const char *pointer);
-char *revstrcasestr(const char *haystack, const char *needle, const char
-	*rev_start);
-char *mbrevstrcasestr(const char *haystack, const char *needle, const
-	char *rev_start);
+	const char *index);
+char *mbrevstrcasestr(const char *haystack, const char *needle,
+	const char *index);
 size_t mbstrlen(const char *s);
 size_t mbstrnlen(const char *s, size_t maxlen);
 #if !defined(NANO_TINY) || !defined(DISABLE_JUSTIFY)
@@ -220,19 +222,16 @@ char *mbstrchr(const char *s, const char *c);
 #endif
 #ifndef NANO_TINY
 char *mbstrpbrk(const char *s, const char *accept);
-char *revstrpbrk(const char *s, const char *accept, const char
-	*rev_start);
-char *mbrevstrpbrk(const char *s, const char *accept, const char
-	*rev_start);
+char *revstrpbrk(const char *head, const char *accept, const char *index);
+char *mbrevstrpbrk(const char *head, const char *accept, const char *index);
 #endif
-#if !defined(DISABLE_NANORC) && (!defined(NANO_TINY) || !defined(DISABLE_JUSTIFY))
-bool has_blank_chars(const char *s);
+#if defined(ENABLE_NANORC) && (!defined(NANO_TINY) || !defined(DISABLE_JUSTIFY))
 bool has_blank_mbchars(const char *s);
 #endif
 #ifdef ENABLE_UTF8
 bool is_valid_unicode(wchar_t wc);
 #endif
-#ifndef DISABLE_NANORC
+#ifdef ENABLE_NANORC
 bool is_valid_mbstring(const char *s);
 #endif
 
@@ -246,14 +245,11 @@ void alloc_multidata_if_needed(filestruct *fileptr);
 void precalc_multicolorinfo(void);
 #endif
 
-/* All functions in cut.c. */
+/* Most functions in cut.c. */
 void cutbuffer_reset(void);
 bool keeping_cutbuffer(void);
-void cut_line(void);
 #ifndef NANO_TINY
 void cut_marked(bool *right_side_up);
-void cut_to_eol(void);
-void cut_to_eof(void);
 #endif
 void do_cut_text(bool copy_text, bool cut_till_eof);
 void do_cut_text_void(void);
@@ -264,7 +260,6 @@ void do_cut_till_eof(void);
 void do_uncut_text(void);
 
 /* Most functions in files.c. */
-void make_new_buffer(void);
 void initialize_buffer_text(void);
 bool open_buffer(const char *filename, bool undoable);
 #ifndef DISABLE_SPELLER
@@ -274,20 +269,18 @@ void replace_marked_buffer(const char *filename, filestruct *top, size_t top_x,
 	filestruct *bot, size_t bot_x);
 #endif
 #endif
-void display_buffer(void);
-#ifndef DISABLE_MULTIBUFFER
+void prepare_for_display(void);
+#ifdef ENABLE_MULTIBUFFER
 void switch_to_prev_buffer_void(void);
 void switch_to_next_buffer_void(void);
 bool close_buffer(void);
 #endif
-char *encode_data(char *buf, size_t buf_len);
 void read_file(FILE *f, int fd, const char *filename, bool undoable,
 		bool checkwritable);
 int open_file(const char *filename, bool newfie, bool quiet, FILE **f);
 char *get_next_filename(const char *name, const char *suffix);
 void do_insertfile_void(void);
 char *get_full_path(const char *origpath);
-char *check_writable_directory(const char *path);
 char *safe_tempfile(FILE **f);
 #ifndef DISABLE_OPERATINGDIR
 void init_operating_dir(void);
@@ -298,7 +291,7 @@ void init_backup_dir(void);
 int delete_lockfile(const char *lockfilename);
 int write_lockfile(const char *lockfilename, const char *origfilename, bool modified);
 #endif
-int copy_file(FILE *inn, FILE *out);
+int copy_file(FILE *inn, FILE *out, bool close_out);
 bool write_file(const char *name, FILE *f_open, bool tmp,
 	kind_of_writing_type method, bool nonamechange);
 #ifndef NANO_TINY
@@ -311,24 +304,17 @@ void do_writeout_void(void);
 void do_savefile(void);
 #endif
 char *real_dir_from_tilde(const char *buf);
-#if !defined(DISABLE_TABCOMP) || !defined(DISABLE_BROWSER)
+#if defined(ENABLE_TABCOMP) || defined(ENABLE_BROWSER)
 int diralphasort(const void *va, const void *vb);
 void free_chararray(char **array, size_t len);
 #endif
-#ifndef DISABLE_TABCOMP
-bool is_dir(const char *buf);
-char **username_tab_completion(const char *buf, size_t *num_matches,
-	size_t buf_len);
-char **cwd_tab_completion(const char *buf, bool allow_files, size_t
-	*num_matches, size_t buf_len);
+#ifdef ENABLE_TABCOMP
 char *input_tab(char *buf, bool allow_files, size_t *place,
 	bool *lastwastab, void (*refresh_func)(void), bool *listed);
 #endif
 const char *tail(const char *path);
 #ifndef DISABLE_HISTORIES
-char *histfilename(void);
 void load_history(void);
-bool writehist(FILE *hist, const filestruct *head);
 void save_history(void);
 int check_dotnano(void);
 void load_poshistory(void);
@@ -358,7 +344,8 @@ void thanks_for_all_the_fish(void);
 #endif
 
 /* All functions in help.c. */
-#ifndef DISABLE_HELP
+#ifdef ENABLE_HELP
+void wrap_the_help_text(bool redisplaying);
 void do_help(void);
 void help_init(void);
 functionptrtype parse_help_input(int *kbinput);
@@ -372,16 +359,16 @@ void do_last_line(void);
 void do_page_up(void);
 void do_page_down(void);
 #ifndef DISABLE_JUSTIFY
-void do_para_begin(bool allow_update);
+void do_para_begin(bool update_screen);
 void do_para_begin_void(void);
-void do_para_end(bool allow_update);
+void do_para_end(bool update_screen);
 void do_para_end_void(void);
 #endif
 void do_prev_block(void);
 void do_next_block(void);
-void do_prev_word(bool allow_punct, bool allow_update);
+void do_prev_word(bool allow_punct, bool update_screen);
 void do_prev_word_void(void);
-bool do_next_word(bool allow_punct, bool allow_update);
+bool do_next_word(bool allow_punct, bool update_screen);
 void do_next_word_void(void);
 void do_home(bool be_clever);
 void do_home_void(void);
@@ -398,9 +385,8 @@ void do_scroll_down(void);
 void do_left(void);
 void do_right(void);
 
-/* All functions in nano.c. */
+/* Most functions in nano.c. */
 filestruct *make_new_node(filestruct *prevnode);
-filestruct *copy_node(const filestruct *src);
 void splice_node(filestruct *afterthis, filestruct *newnode);
 void unlink_node(filestruct *fileptr);
 void delete_node(filestruct *fileptr);
@@ -419,24 +405,15 @@ void unlink_opennode(openfilestruct *fileptr);
 void delete_opennode(openfilestruct *fileptr);
 void print_view_warning(void);
 void show_restricted_warning(void);
-#ifdef DISABLE_HELP
+#ifndef ENABLE_HELP
 void say_there_is_no_help(void);
 #endif
 void finish(void);
 void die(const char *msg, ...);
 void die_save_file(const char *die_filename, struct stat *die_stat);
 void window_init(void);
-#ifndef DISABLE_MOUSE
-void disable_mouse_support(void);
-void enable_mouse_support(void);
-void mouse_init(void);
-#endif
-void print_opt(const char *shortflag, const char *longflag, const char *desc);
-void usage(void);
-void version(void);
 void do_exit(void);
 void close_and_go(void);
-void signal_init(void);
 RETSIGTYPE handle_hupterm(int signal);
 RETSIGTYPE do_suspend(int signal);
 RETSIGTYPE do_continue(int signal);
@@ -453,13 +430,13 @@ void enable_flow_control(void);
 void terminal_init(void);
 void unbound_key(int code);
 int do_input(bool allow_funcs);
-#ifndef DISABLE_MOUSE
+#ifdef ENABLE_MOUSE
 int do_mouse(void);
 #endif
 void do_output(char *output, size_t output_len, bool allow_cntrls);
 
 /* Most functions in prompt.c. */
-#ifndef DISABLE_MOUSE
+#ifdef ENABLE_MOUSE
 int do_statusbar_mouse(void);
 #endif
 void do_statusbar_output(int *the_input, size_t input_len,
@@ -479,7 +456,6 @@ void do_statusbar_verbatim_input(void);
 size_t statusbar_xplustabs(void);
 size_t get_statusbar_page_start(size_t start_col, size_t column);
 void reinit_statusbar_x(void);
-void reset_statusbar_cursor(void);
 void update_the_statusbar(void);
 int do_prompt(bool allow_tabs, bool allow_files,
 	int menu, const char *curranswer,
@@ -490,24 +466,18 @@ int do_prompt(bool allow_tabs, bool allow_files,
 int do_yesno_prompt(bool all, const char *msg);
 
 /* Most functions in rcfile.c. */
-#if !defined(DISABLE_NANORC) || !defined(DISABLE_HISTORIES)
-char *parse_next_word(char *ptr);
-#endif
-#ifndef DISABLE_NANORC
+#ifdef ENABLE_NANORC
 #ifndef DISABLE_COLOR
 bool parse_color_names(char *combostr, short *fg, short *bg, bool *bright);
 void grab_and_store(const char *kind, char *ptr, regexlisttype **storage);
 #endif
 void parse_rcfile(FILE *rcstream, bool syntax_only);
 void do_rcfiles(void);
-#endif /* !DISABLE_NANORC */
+#endif /* ENABLE_NANORC */
 
-/* All functions in search.c. */
-bool regexp_init(const char *regexp);
-void regexp_cleanup(void);
+/* Most functions in search.c. */
 void not_found_msg(const char *str);
 void search_replace_abort(void);
-int search_init(bool replacing, bool use_answer);
 int findnextstr(const char *needle, bool whole_word_only, bool have_region,
 	size_t *match_len, bool skipone, const filestruct *begin, size_t begin_x);
 void do_search(void);
@@ -517,8 +487,6 @@ void do_findnext(void);
 #endif
 void do_research(void);
 void go_looking(void);
-int replace_regexp(char *string, bool create);
-char *replace_line(const char *needle);
 ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 	const filestruct *real_current, size_t *real_current_x);
 void do_replace(void);
@@ -527,9 +495,8 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 	bool interactive);
 void do_gotolinecolumn_void(void);
 #ifndef NANO_TINY
-bool find_bracket_match(bool reverse, const char *bracket_set);
 void do_find_bracket(void);
-#ifndef DISABLE_TABCOMP
+#ifdef ENABLE_TABCOMP
 char *get_history_completion(filestruct **h, char *s, size_t len);
 #endif
 #endif
@@ -537,8 +504,6 @@ char *get_history_completion(filestruct **h, char *s, size_t len);
 bool history_has_changed(void);
 void history_init(void);
 void history_reset(const filestruct *h);
-filestruct *find_history(const filestruct *h_start, const filestruct
-	*h_end, const char *s, size_t len);
 void update_history(filestruct **h, const char *s);
 char *get_history_older(filestruct **h);
 char *get_history_newer(filestruct **h);
@@ -546,7 +511,7 @@ void get_history_older_void(void);
 void get_history_newer_void(void);
 #endif
 
-/* All functions in text.c. */
+/* Most functions in text.c. */
 #ifndef NANO_TINY
 void do_mark(void);
 #endif
@@ -584,7 +549,7 @@ void update_undo(undo_type action);
 void wrap_reset(void);
 bool do_wrap(filestruct *line);
 #endif
-#if !defined(DISABLE_HELP) || !defined(DISABLE_WRAPJUSTIFY)
+#if defined(ENABLE_HELP) || !defined(DISABLE_WRAPJUSTIFY)
 ssize_t break_line(const char *line, ssize_t goal, bool snap_at_nl);
 #endif
 #if !defined(NANO_TINY) || !defined(DISABLE_JUSTIFY)
@@ -592,23 +557,13 @@ size_t indent_length(const char *line);
 #endif
 #ifndef DISABLE_JUSTIFY
 void justify_format(filestruct *paragraph, size_t skip);
-size_t quote_length(const char *line);
-bool quotes_match(const char *a_line, size_t a_quote, const char
-	*b_line);
-bool indents_match(const char *a_line, size_t a_indent, const char
-	*b_line, size_t b_indent);
 bool begpar(const filestruct *const foo);
 bool inpar(const filestruct *const foo);
-void backup_lines(filestruct *first_line, size_t par_len);
-bool find_paragraph(size_t *const quote, size_t *const par);
 void do_justify(bool full_justify);
 void do_justify_void(void);
 void do_full_justify(void);
 #endif
 #ifndef DISABLE_SPELLER
-bool do_int_spell_fix(const char *word);
-const char *do_int_speller(const char *tempfile_name);
-const char *do_alt_speller(char *tempfile_name);
 void do_spell(void);
 #endif
 #ifndef DISABLE_COLOR
@@ -650,8 +605,10 @@ size_t actual_x(const char *s, size_t column);
 size_t strnlenpt(const char *s, size_t maxlen);
 size_t strlenpt(const char *s);
 void new_magicline(void);
-#ifndef NANO_TINY
+#if !defined(NANO_TINY) || defined(ENABLE_HELP)
 void remove_magicline(void);
+#endif
+#ifndef NANO_TINY
 void mark_order(const filestruct **top, size_t *top_x, const filestruct
 	**bot, size_t *bot_x, bool *right_side_up);
 #endif
@@ -667,34 +624,25 @@ void dump_filestruct_reverse(void);
 /* Most functions in winio.c. */
 void get_key_buffer(WINDOW *win);
 size_t get_key_buffer_len(void);
-void unget_input(int *input, size_t input_len);
 void unget_kbinput(int kbinput, bool metakey);
-int *get_input(WINDOW *win, size_t input_len);
 int get_kbinput(WINDOW *win);
 int parse_kbinput(WINDOW *win);
 int arrow_from_abcd(int kbinput);
 int parse_escape_sequence(WINDOW *win, int kbinput);
 int get_byte_kbinput(int kbinput);
-#ifdef ENABLE_UTF8
-long add_unicode_digit(int kbinput, long factor, long *uni);
-long get_unicode_kbinput(WINDOW *win, int kbinput);
-#endif
 int get_control_kbinput(int kbinput);
-void unparse_kbinput(char *output, size_t output_len);
 int *get_verbatim_kbinput(WINDOW *win, size_t *kbinput_len);
 int *parse_verbatim_kbinput(WINDOW *win, size_t *count);
-#ifndef DISABLE_MOUSE
+#ifdef ENABLE_MOUSE
 int get_mouseinput(int *mouse_x, int *mouse_y, bool allow_shortcuts);
 #endif
 const sc *get_shortcut(int *kbinput);
 void blank_row(WINDOW *win, int y, int x, int n);
-void blank_titlebar(void);
 void blank_edit(void);
 void blank_statusbar(void);
 void blank_bottombars(void);
 void check_statusblank(void);
-char *display_string(const char *buf, size_t start_col, size_t span,
-	bool dollars);
+char *display_string(const char *buf, size_t column, size_t span, bool isdata);
 void titlebar(const char *path);
 extern void set_modified(void);
 void statusbar(const char *msg);
@@ -702,7 +650,7 @@ void warn_and_shortly_pause(const char *msg);
 void statusline(message_type importance, const char *msg, ...);
 void bottombars(int menu);
 void onekey(const char *keystroke, const char *desc, int length);
-void reset_cursor(void);
+void place_the_cursor(bool forreal);
 void edit_draw(filestruct *fileptr, const char *converted,
 	int line, size_t from_col);
 int update_line(filestruct *fileptr, size_t index);
@@ -714,10 +662,9 @@ int go_back_chunks(int nrows, filestruct **line, size_t *leftedge);
 int go_forward_chunks(int nrows, filestruct **line, size_t *leftedge);
 bool less_than_a_screenful(size_t was_lineno, size_t was_leftedge);
 void edit_scroll(scroll_dir direction, int nrows);
+#ifndef NANO_TINY
 void ensure_firstcolumn_is_aligned(void);
-bool current_is_above_screen(void);
-bool current_is_below_screen(void);
-bool current_is_offscreen(void);
+#endif
 void edit_redraw(filestruct *old_current);
 void edit_refresh(void);
 void adjust_viewport(update_type location);
@@ -730,28 +677,34 @@ void spotlight(bool active, const char *word);
 void xon_complaint(void);
 void xoff_complaint(void);
 void do_suspend_void(void);
-void enable_nodelay(void);
-void disable_nodelay(void);
+void disable_waiting(void);
+void enable_waiting(void);
 #ifndef DISABLE_EXTRA
 void do_credits(void);
 #endif
 
-/* May as well throw these here, since they are just placeholders. */
+/* These are just name definitions. */
 void do_cancel(void);
 void case_sens_void(void);
 void regexp_void(void);
+void backwards_void(void);
+void flip_replace(void);
 void gototext_void(void);
+#ifdef ENABLE_BROWSER
 void to_files_void(void);
+void goto_dir_void(void);
+#endif
+#ifndef NANO_TINY
 void dos_format_void(void);
 void mac_format_void(void);
 void append_void(void);
 void prepend_void(void);
 void backup_file_void(void);
+void flip_execute(void);
+#endif
+#ifdef ENABLE_MULTIBUFFER
+void flip_newbuffer(void);
+#endif
 void discard_buffer(void);
-void new_buffer_void(void);
-void backwards_void(void);
-void goto_dir_void(void);
-void flip_replace_void(void);
-void flip_execute_void(void);
 
 #endif /* !PROTO_H */
